# frozen_string_literal: true

#
# Cookbook Name:: snu-sumologic
# Library:: resource/snu_sumologic_collector
#
# Copyright (C) 2014-2017, Socrata, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

require 'chef/resource'

class Chef
  class Resource
    # The sumologic_collector resource uses the LWRP DSL, which means we can't
    # easily subclass it and need to wrap a new resource around it instead.
    #
    # @author Jonathan Hartman <jonathan.hartman@socrata.com>
    class SnuSumologicCollector < Resource
      provides :snu_sumologic_collector

      default_action :install_and_configure

      # Ensure the Sumo credentials don't leak to any logs.
      property :sumo_access_id, String, sensitive: true
      property :sumo_access_key, String, sensitive: true

      # Use the installDir attribute by default for the dir.
      property :dir, String, default: '/opt/SumoCollector'

      # Use the node's FQDN as the default collector name.
      property :collector_name, String, default: lazy { node.name }

      # Use the .d dir in the JSON path as the sync source.
      property :sync_sources, String, default: '/etc/sumo.d'

      # We're using the cloud so collectors should default to ephemeral.
      property :ephemeral, [TrueClass, FalseClass], default: true

      # The collector name includes a unique instance ID, so it's safe to
      # clobber if the same collector re-registers itself.
      property :clobber, [TrueClass, FalseClass], default: true

      # Default the runas_username to root. If we don't specify a user, the
      # sumo installer has trouble when run from a Chef process that's not a
      # descendent of a login shell.
      property :runas_username, String, default: 'root'

      # The Java heap sizes should default to 256.
      property :wrapper_java_initmemory, Integer, default: 256
      property :wrapper_java_maxmemory, Integer, default: 256

      # Save a sources property so we can track accumulated sources for the
      # :manage action.
      property :sources, Array, default: []

      # Provide a place to store any properties declared by #method_missing.
      property :additional_properties, Hash, default: {}

      # Pass any other property methods on to the underlying
      # sumologic_collector resource so we don't have to keep track of what it
      # supports.
      #
      # (see Chef::Resource#method_missing)
      def method_missing(method_symbol, *args, &block)
        super
      rescue NoMethodError
        raise if !block.nil? || args.length != 1
        new_props = additional_properties.merge(method_symbol => args[0])
        additional_properties(new_props)
      end

      # We don't need anything special out of respond_to_missing? since all the
      # missing methods we allow get saved into additional_properties.
      #
      # (see Object#respond_to_missing)
      def respond_to_missing?(method_symbol, include_private = false)
        super
      end

      # Load the existing sources as defined on the filesystem. Every other
      # property is handled by file/template/etc. sub-resources for which the
      # current value doesn't need to be loaded.
      load_current_value do |desired|
        srcs = if Dir.exist?(desired.sync_sources)
                 (Dir.entries(desired.sync_sources) - %w[. ..]).map do |i|
                   i.gsub(/\.json$/, '')
                 end
               else
                 []
               end
        sources(srcs)
      end

      # Wrap the :install and :install_and_configure actions to ensure the .d
      # config directory is created.
      %i[install install_and_configure].each do |act|
        action act do
          rc = run_context.parent_run_context.nil? ? :root : :parent
          with_run_context rc do
            directory new_resource.sync_sources do
              owner 'root'
              group 'root'
              mode '0755'
              recursive true
            end

            sumologic_collector new_resource.name do
              new_resource.class.state_properties.map(&:name).each do |prop|
                next if %i[sources additional_properties].include?(prop)
                next if new_resource.send(prop).nil?

                send(prop, new_resource.send(prop))
              end

              new_resource.additional_properties.each do |k, v|
                send(k, v) unless v.nil?
              end

              action act
            end
          end
        end
      end

      # Every other action should just be passed on to the sumologic_collector
      # resource.
      %i[configure remove start stop restart enable disable].each do |act|
        action act do
          rc = run_context.parent_run_context.nil? ? :root : :parent
          with_run_context rc do
            sumologic_collector new_resource.name do
              new_resource.class.state_properties.map(&:name).each do |prop|
                next if %i[sources additional_properties].include?(prop)
                next if new_resource.send(prop).nil?

                send(prop, new_resource.send(prop))
              end

              new_resource.additional_properties.each do |k, v|
                send(k, v) unless v.nil?
              end

              action act
            end
          end
        end
      end

      # Go over every Sumo source configured on the filesystem and delete any
      # that are (no longer) part of our resource collection.
      #
      # This should only be used as part of a delayed action or notification,
      # otherwise it may delete sources unexpectedly depending on the order of
      # resource collection.
      action :manage do
        (current_resource.sources - new_resource.sources).each do |src|
          snu_sumo_source_local_file src do
            source_json_directory new_resource.sync_sources
            action :delete
          end
        end
      end
    end
  end
end
